@@ -53,15 +53,14 @@ sub close {
 }
 
 sub send_request_and_receive_response {
-    my ($self, $call_type, $object, $method, $params, $opts) = @_;
-    my $wantarray = wantarray;
-
+    my $self = shift;
+    
     if ($RMI::DEBUG) {
-        print "$RMI::DEBUG_MSG_PREFIX N: $$ calling via $self on $object: $method with @$params\n";
+        print "$RMI::DEBUG_MSG_PREFIX N: $$ calling via $self: @_\n";
     }
     
-    $self->_send('query',[$method,$wantarray,$object,($params ? @$params : ())])
-                or die "failed to send! $!";
+    my $wantarray = wantarray;
+    $self->_send('query',[$wantarray, @_]) or die "failed to send! $!";
     
     for (1) {
         my ($message_type, $message_data) = $self->_receive();
@@ -102,7 +101,7 @@ sub receive_request_and_send_response {
         return;
     }
     else {
-        die "Unexpected message type $message_type!  message_data was:" . message_data::Dumper::Dumper($message_data);
+        die "Unexpected message type $message_type!  message_data was:" . Dumper::Dumper($message_data);
     }        
 }
 
@@ -138,58 +137,42 @@ sub _receive {
 
 sub _process_query {
     my ($self, $message_data) = @_;
-    my ($method, $wantarray, $object, @params) = @$message_data;
+
+    my $wantarray = shift @$message_data;
+    my $call_type = shift @$message_data;
     
     do {    
         no warnings;
-        print "$RMI::DEBUG_MSG_PREFIX N: $$ unserialized object $object and params: @params\n" if $RMI::DEBUG;
+        print "$RMI::DEBUG_MSG_PREFIX N: $$ processing query $call_type in wantarray context $wantarray with : @$message_data\n" if $RMI::DEBUG;
     };
     
-    push @RMI::executing_nodes, $self;
+    
+    # swap call_ for _respond_to_
+    my $method = '_respond_to_' . substr($call_type,5);
     
     my @result;
+
+    push @RMI::executing_nodes, $self;
     eval {
-        if (defined $object) {
-            #eval "use $object"; if not ref($object);
-            if (not defined $wantarray) {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with undef wantarray\n" if $RMI::DEBUG;
-                $object->$method(@params);
-            }
-            elsif ($wantarray) {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with true wantarray\n" if $RMI::DEBUG;
-                @result = $object->$method(@params);
-            }
-            else {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with false wantarray\n" if $RMI::DEBUG;
-                my $result = $object->$method(@params);
-                @result = ($result);
-            }
+        if (not defined $wantarray) {
+            print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with undef wantarray\n" if $RMI::DEBUG;
+            $self->$method(@$message_data);
+        }
+        elsif ($wantarray) {
+            print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with true wantarray\n" if $RMI::DEBUG;
+            @result = $self->$method(@$message_data);
         }
         else {
-            no strict 'refs';
-            if (not defined $wantarray) {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ function call with undef wantarray\n" if $RMI::DEBUG;                            
-                $method->(@params);
-            }
-            elsif ($wantarray) {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ function call with true wantarray\n" if $RMI::DEBUG;                
-                @result = $method->(@params);
-            }
-            else {
-                print "$RMI::DEBUG_MSG_PREFIX N: $$ function call with false wantarray\n" if $RMI::DEBUG;                
-                my $result = $method->(@params);
-                @result = ($result);
-            }
+            print "$RMI::DEBUG_MSG_PREFIX N: $$ object call with false wantarray\n" if $RMI::DEBUG;
+            my $result = $self->$method(@$message_data);
+            @result = ($result);
         }
     };
-    
     pop @RMI::executing_nodes;
 
     # we MUST undef these in case they are the only references to remote objects which need to be destroyed
     # the DESTROY handler will queue them for deletion, and _send() will include them in the message to the other side
     @$message_data = ();
-    $object = undef;
-    @params = undef;
     
     my ($return_type, $return_data);
     if ($@) {
@@ -205,7 +188,106 @@ sub _process_query {
     return ($return_type, $return_data);
 }
 
+# private API for the server-ish role
+
+sub _respond_to_function {
+    my ($self, $fname, @params) = @_;
+    no strict 'refs';
+    $fname->(@params);
+}
+
+sub _respond_to_class_method {
+    my ($self, $class, $method, @params) = @_;
+    $class->$method(@params);
+}
+
+sub _respond_to_object_method {
+    my ($self, $object, $method, @params) = @_;
+    $object->$method(@params);
+}
+
+sub _respond_to_use {
+    my ($self,$class,$module,$has_args,@use_args) = @_;
+
+    no strict 'refs';
+    if ($class and not $module) {
+        $module = $class;
+        $module =~ s/::/\//g;
+        $module .= '.pm';
+    }
+    elsif ($module and not $class) {
+        $class = $module;
+        $class =~ s/\//::/g;
+        $class =~ s/.pm$//; 
+    }
+    
+    my $n = $RMI::Exported::count++;
+    my $tmp_package_to_catch_exports = 'RMI::Exported::P' . $n;
+    my $src = "
+        package $tmp_package_to_catch_exports;
+        require $class;
+        my \@exports = ();
+        if (\$has_args) {
+            if (\@use_args) {
+                $class->import(\@use_args);
+                \@exports = grep { ${tmp_package_to_catch_exports}->can(\$_) } keys \%${tmp_package_to_catch_exports}::;
+            }
+            else {
+                # print qq/no import because of empty list!/;
+            }
+        }
+        else {
+            $class->import();
+            \@exports = grep { ${tmp_package_to_catch_exports}->can(\$_) } keys \%${tmp_package_to_catch_exports}::;
+        }
+        return (\$INC{'$module'}, \@exports);
+    ";
+    my ($path, @exported) = eval($src);
+    die $@ if $@;
+    return ($class,$module,$path,@exported);
+}
+
+sub _respond_to_use_lib {
+    my $self = shift; 
+    my $lib = shift;
+    require lib;
+    return lib->import($lib);
+}
+
+sub _respond_to_eval {
+    my $self = shift;
+    my $src = shift;
+    if (wantarray) {
+        my @result = eval $src;
+        die $@ if $@;
+        return @result;        
+    }
+    else {
+        my $result = eval $src;
+        die $@ if $@;
+        return $result;
+    }
+}
+
+sub _respond_to_coderef {
+    # This is used when a CODE ref is proxied, since you can't tie CODE refs.
+    # It does not have a matching caller in RMI::Client.
+    # The other reference types are handled by "tie" to RMI::ProxyReferecnce.
+
+    # NOTE: It's important to shift these two parameters off since goto must 
+    # pass the remainder of @_ to the subroutine.
+    my $self = shift;
+    my $sub_id = shift;
+    my $sub = $self->{_sent_objects}{$sub_id};
+    die "$sub is not a CODE ref.  came from $sub_id\n" unless $sub and ref($sub) eq 'CODE';
+    goto $sub;
+}
+
+# The private API for the client-ish role of the RMI::Node is still in the RMI::Client module,
+# where it is documented.  All of that API is a thin wrapper for methods here.
+
 # serialize params when sending a query, or results when sending a response
+
 sub _serialize {
     my ($self, $message_type, $message_data) = @_;    
     
@@ -278,6 +360,7 @@ sub _serialize {
 }
 
 # deserialize params when receiving a query, or results when receiving a response
+
 sub _deserialize {
     my ($self, $serialized_blob) = @_;
     
@@ -333,7 +416,7 @@ sub _deserialize {
                 elsif ($remote_shape eq 'CODE') {
                     my $sub_id = $value;
                     $o = sub {
-                        $self->send_request_and_receive_response('call_coderef', undef, 'RMI::Node::_exec_coderef', [$sub_id, @_]);
+                        $self->send_request_and_receive_response('call_coderef', $sub_id, @_);
                     };
                     # TODO: ensure this cleans up on the other side when it is destroyed
                 }
@@ -386,15 +469,77 @@ sub _deserialize {
     return ($message_type,\@message_data);
 }
 
-# This is used when a CODE ref is proxied, since you can't tie CODE refs.
-# It is in this class instead of the server, a coderef could be sent to the
-# server, causing the server to counter-query the client.
+# this proxies a single variable
 
-sub _exec_coderef {
-    my $sub_id = shift;
-    my $sub = $RMI::executing_nodes[-1]{_sent_objects}{$sub_id};
-    die "$sub is not a CODE ref.  came from $sub_id\n" unless $sub and ref($sub) eq 'CODE';
-    goto $sub;
+sub bind_local_var_to_remote {
+    my $self = shift;
+    my $local_var = shift;
+    my $remote_var = (@_ ? shift : $local_var);
+    
+    my $type = substr($local_var,0,1);
+    if (index($local_var,'::')) {
+        $local_var = substr($local_var,1);
+    }
+    else {
+        my $caller = caller();
+        $local_var = $caller . '::' . substr($local_var,1);
+    }
+
+    unless ($type eq substr($remote_var,0,1)) {
+        die "type mismatch: local var $local_var has type $type, while remote is $remote_var!";
+    }
+    if (index($remote_var,'::')) {
+        $remote_var = substr($remote_var,1);
+    }
+    else {
+        my $caller = caller();
+        $remote_var = $caller . '::' . substr($remote_var,1);
+    }
+    
+    my $src = '\\' . $type . $remote_var . ";\n";
+    my $r = $self->call_eval($src);
+    die $@ if $@;
+    $src = '*' . $local_var . ' = $r' . ";\n";
+    eval $src;
+    die $@ if $@;
+    return 1;
+}
+
+# this proxies an entire class instead of just a single object
+
+sub bind_local_class_to_remote {
+    my $self = shift;
+    my ($class,$module,$path,@exported) = $self->call_use(@_);
+    my $re_bind = 0;
+    if (my $prior = $RMI::proxied_classes{$class}) {
+        if ($prior != $self) {
+            die "class $class has already been proxied by another RMI client: $prior!";
+        }
+        else {
+            # re-binding a class to the same remote side doesn't hurt,
+            # and allowing it allows the effect of export to occur
+            # in multiple places on the client side.
+        }
+    }
+    elsif (my $path = $INC{$module}) {
+        die "module $module has already been used locally from path: $path";
+    }
+    no strict 'refs';
+    for my $sub (qw/AUTOLOAD DESTROY can isa/) {
+        *{$class . '::' . $sub} = \&{ 'RMI::ProxyObject::' . $sub }
+    }
+    if (@exported) {
+        my $caller ||= caller(0);
+        if (substr($caller,0,5) eq 'RMI::') { $caller = caller(1) }
+        for my $sub (@exported) {
+            my @pair = ('&' . $caller . '::' . $sub => '&' . $class . '::' . $sub);
+            print "$RMI::DEBUG_MSG_PREFIX N: $$ bind pair $pair[0] $pair[1]\n" if $RMI::DEBUG;
+            $self->bind_local_var_to_remote(@pair);
+        }
+    }
+    $RMI::proxied_classes{$class} = $self;
+    $INC{$module} = $self;
+    print "$class used remotely via $self.  Module $module found at $path remotely.\n" if $RMI::DEBUG;    
 }
 
 # used for testing
@@ -402,13 +547,13 @@ sub _exec_coderef {
 sub _remote_has_ref {
     my ($self,$obj) = @_;
     my $id = "$obj";
-    my $has_sent = $self->send_request_and_receive_response('call_eval', undef, "RMI::Server::_receive_eval", ['exists $RMI::executing_nodes[-1]->{_received_objects}{"' . $id . '"}']);
+    my $has_sent = $self->send_request_and_receive_response('call_eval', 'exists $RMI::executing_nodes[-1]->{_received_objects}{"' . $id . '"}');
 }
 
 sub _remote_has_sent {
     my ($self,$obj) = @_;
     my $id = "$obj";
-    my $has_sent = $self->send_request_and_receive_response('call_eval', undef, "RMI::Server::_receive_eval", ['exists $RMI::executing_nodes[-1]->{_sent_objects}{"' . $id . '"}']);
+    my $has_sent = $self->send_request_and_receive_response('call_eval', 'exists $RMI::executing_nodes[-1]->{_sent_objects}{"' . $id . '"}');
 }
 
 # this generate basic accessors w/o using any other Perl modules which might have proxy effects
@@ -460,7 +605,7 @@ RMI::Node - base class for RMI::Client and RMI::Server
     }
     
     # send one request and get the result
-    $sum = $c->send_request_and_receive_response('call_function', undef, 'main::add', 5, 6);
+    $sum = $c->send_request_and_receive_response('call_function', 'main::add', 5, 6);
     
     # we might have also done..
     $robj = $c->send_request_and_receive_response('call_class_method', 'IO::File', 'new', '/my/file');
@@ -517,8 +662,7 @@ This is the primary method used by nodes acting in a client-like capacity.
  $call_type:    one of: call_object_method, call_class_method, or call_function, or one of several internal types
  $object:       the object or class on which the method is being called, may be undef for subroutine/function calls
  $method:       the method to call on $object (even if $object is a class name), or the fully-qualified sub name
- $params:       an optional arrayref of values which should be passed to $method
- $opts:         an optional hashref of values which can control/optimize message passing
+ @params:       an optional array of values which should be passed to $method
 
 Return values:
 
@@ -566,20 +710,20 @@ This is the type for standard remote method invocatons.
   
 The message data contains, in order:
 
-
- - method_name  This is the name of the method to call.
-                This is a fully-qualified function name for plain function calls.
-
  - wantarray    1, '', or undef, depending on the requestor's calling context.
                 This is passed to the remote side, and also used on the
                 local side to control how results are returned.
 
- - object       A class name, or an object which is a proxy for something on the remote side.
-                This value is undef for plain function calls.
+ - object/class A class name, or an object which is a proxy for something on the remote side.
+                This value is not present for plain function calls, or evals.
 
- - param1       The first parameter to the function/method call
+ - method_name  This is the name of the method to call.
+                This is a fully-qualified function name for plain function calls.
 
- - ...          The next parameter to the function/method call
+ - param1       The first parameter to the function/method call.
+                Note that parameters are "passed" to eval as well by exposing @_.
+
+ - ...          The next parameter to the function/method call, etc.
 
 
 =head2 result
